/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @file argos2/common/utility/math/general.h
 *
 * @brief Contains the definition of some basic math functions.
 *
 * @author Carlo Pinciroli <cpinciro@ulb.ac.be>
 */

#ifndef GENERAL_H
#define GENERAL_H

#include <argos2/common/utility/datatypes/datatypes.h>

namespace argos {

   /****************************************/
   /****************************************/

   /**
    * Returns the absolute value of the passed argument.
    * @return the absolute value of the passed argument.
    */
   template<typename T> T Abs(const T& t_v) {
      if(t_v > T(0)) return t_v;
      if(t_v < T(0)) return -t_v;
      return T(0);
   }

   /**
    * Returns the absolute value of the passed argument.
    * Specific implementation for the SInt32 type.
    * @return the absolute value of the passed argument.
    * @see Abs()
    */
   inline SInt32 Abs(SInt32 t_v) {
      if(t_v > 0) return t_v;
      if(t_v < 0) return -t_v;
      return 0;
   }

   /**
    * Returns the absolute value of the passed argument.
    * Specific implementation for the Real type.
    * @return the absolute value of the passed argument.
    */
   inline Real Abs(Real t_v) {
      if(t_v > 0.0f) return t_v;
      if(t_v < 0.0f) return -t_v;
      return 0.0f;
   }

   /****************************************/
   /****************************************/

#ifdef ARGOS_DOUBLE_PRECISION
#  define Log  ::log
#  define Sqrt ::sqrt
#else
#  define Log ::logf
#  define Sqrt ::sqrtf
#endif

   /****************************************/
   /****************************************/

   /**
    * Returns the smaller of the two passed arguments.
    * This version accepts <tt>const</tt> references as arguments.
    * @return the smaller of the two passed arguments.
    */
   template<typename T> T Min(const T& t_v1, const T& t_v2) {
      return t_v1 < t_v2 ? t_v1 : t_v2;
   }

   /**
    * Returns the smaller of the two passed arguments.
    * This version accepts non-<tt>const</tt> references as arguments.
    * @return the smaller of the two passed arguments.
    */
   template<typename T> T& Min(T& t_v1, T& t_v2) {
      return t_v1 < t_v2 ? t_v1 : t_v2;
   }

   /**
    * Returns the bigger of the two passed arguments.
    * This version accepts <tt>const</tt> references as arguments.
    * @return the bigger of the two passed arguments.
    */
   template<typename T> T Max(const T& t_v1, const T& t_v2) {
      return t_v1 > t_v2 ? t_v1 : t_v2;
   }

   /**
    * Returns the bigger of the two passed arguments.
    * This version accepts non-<tt>const</tt> references as arguments.
    * @return the bigger of the two passed arguments.
    */
   template<typename T> T& Max(T& t_v1, T& t_v2) {
      return t_v1 > t_v2 ? t_v1 : t_v2;
   }

   /****************************************/
   /****************************************/

   /**
    * Returns the sign of the value of the passed argument.
    * @return -1 if the value is negative, +1 if it is positive, or 0 if the value is 0.
    */
   template<typename T> SInt32 Sign(const T& t_v) {
      if(t_v > T(0)) return 1;
      if(t_v < T(0)) return -1;
      return 0;
   }

   /****************************************/
   /****************************************/

   /**
    * Returns the square of the value of the passed argument.
    * @return the square of the value of the passed argument.
    */
   template<typename T> T Square(const T& t_v) {
      return t_v * t_v;
   }

   /****************************************/
   /****************************************/

   /**
    * Rounds the passed floating-point value to the closest lower integer.
    * Profiled, it takes 50% the time of ::floor().
    * @return the closest lower integer to the passed value.
    */
   inline SInt32 Floor(Real f_value) {
      SInt32 nI = static_cast<SInt32>(f_value);
      if(f_value >= 0.0f) return nI;
      return nI - 1;
   }

   /**
    * Rounds the passed floating-point value to the closest higher integer.
    * Profiled, it takes 50% the time of ::ceil().
    * @return the closest higher integer to the passed value.
    */
   inline SInt32 Ceil(Real f_value) {
      SInt32 nI = static_cast<SInt32>(f_value);
      if(nI < f_value) return nI + 1;
      return nI;
   }

   /**
    * Rounds the passed floating-point value to the integer closest to zero.
    * If the passed value is >0, Floor() is called. Otherwise, Ceil() is called.
    * @return the integer closest to zero.
    */
   inline SInt32 RoundClosestToZero(Real f_value) {
      if(f_value > 0.0f) return Floor(f_value);
      return Ceil(f_value);
   }

   /****************************************/
   /****************************************/

   /**
    * Tests whether a floating-point value is lower than another.
    * This function perform the following test:
    * <pre>
    *           Abs(f_value1 - f_value2)
    * --------------------------------------------- <= f_epsilon
    * Max (1.0f, Max(Abs(f_value1), Abs(f_value2)))
    * </pre>
    * @param f_value1 the first value to test
    * @param f_value2 the second value to test
    * @param f_epsilon the tolerance of the test
    * @return <tt>true</tt> if <tt>f_value1</tt> &lt; <tt>f_value2</tt>, false otherwise.
    * @see DoubleEq()
    */
   inline bool DoubleEqAbsolute(Real f_value1, Real f_value2, Real f_epsilon ){
      return Abs<Real>(f_value1 - f_value2) <= f_epsilon * Max<Real> (1.0f, Max<Real>(Abs<Real>(f_value1), Abs<Real>(f_value2)));
   }

   /**
    * Tests whether a floating-point value is lower than another.
    * This method is like DoubleEqAbsolute(), but with <tt>f_epsilon = 0.0001</tt>.
    * @param f_value1 the first value to test
    * @param f_value2 the second value to test
    * @return <tt>true</tt> if <tt>f_value1</tt> &lt; <tt>f_value2</tt>, false otherwise.    
    * @see DoubleEqAbsolute()
    */
   inline bool DoubleEq(Real f_value1, Real f_value2){
      return Abs<Real>(f_value1 - f_value2) <= 0.0001f * Max<Real>(1.0f, Max<Real>(Abs<Real>(f_value1), Abs<Real>(f_value2)));
   }

   /****************************************/
   /****************************************/

}

#endif
